import 'package:flutter/material.dart';
import 'package:flutter_techlify_core/flutter_techlify_core.dart';
import 'package:infinite_scroll_pagination/infinite_scroll_pagination.dart';

class PaginationController {
  int pageKey;
  Function(int, List<dynamic>) paginateController;
  PaginationController({
    this.pageKey,
    this.paginateController,
  });
}

typedef FetchData = Function(
  PaginationController controller,
);

typedef WidgetBuilder<T> = Widget Function(
    BuildContext context, T item, int index);

class PaginatedList<T> extends StatefulWidget {
  PaginatedList(
      {Key key,
      @required this.fetchData,
      this.emptyMessage,
      @required this.itemBuilder,
      @required this.controller})
      : super(key: key);
  final PagingController controller;
  final FetchData fetchData;
  final WidgetBuilder<T> itemBuilder;
  final String emptyMessage;

  @override
  _PaginatedListState<T> createState() => _PaginatedListState<T>();
}

class _PaginatedListState<T> extends State<PaginatedList<T>> {
  PagingController _pagingController;
  int currentPage = 1;

  @override
  void initState() {
    super.initState();
    _pagingController = widget.controller;
    _pagingController.addPageRequestListener((pageKey) {
      currentPage = pageKey;
      widget.fetchData(
          PaginationController(pageKey: pageKey, paginateController: paginate));
    });
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  void updateList() {
    _pagingController.refresh();
  }

  void paginate(int lastPage, List<T> items) {
    final isLastPage = currentPage < lastPage;
    if (isLastPage) {
      final nextPageKey = ++currentPage;
      _pagingController.appendPage(items, nextPageKey);
    } else {
      _pagingController.appendLastPage(items);
    }
  }

  @override
  Widget build(BuildContext context) {
    return PagedListView<int, T>(
      physics: NeverScrollableScrollPhysics(),
      shrinkWrap: true,
      pagingController: _pagingController,
      builderDelegate: PagedChildBuilderDelegate<T>(
        noMoreItemsIndicatorBuilder: (context) => Padding(
          padding: const EdgeInsets.all(8.0),
          child: Center(
              child: Text('..That\'s all..',
                  style: TextStyle(color: Colors.grey, fontSize: 12))),
        ),
        itemBuilder: (context, item, index) =>
            widget.itemBuilder(context, item, index),
        firstPageErrorIndicatorBuilder: (context) => EmptyList(
          message: _pagingController.error,
        ),
        newPageProgressIndicatorBuilder: (context) => Spinner(),
        firstPageProgressIndicatorBuilder: (context) => LoaderAnimation(),
        animateTransitions: true,
        noItemsFoundIndicatorBuilder: (context) => EmptyList(
          message: widget.emptyMessage,
        ),
      ),
    );
  }
}
